<?php

class ExtendedPoll_Extend_DataWriter_Poll extends XFCP_ExtendedPoll_Extend_DataWriter_Poll
{
	protected function _getFields()
	{
		$fields = parent::_getFields();

		$fields['xf_poll']['extendedpoll_options'] = array(
			'type' => XenForo_DataWriter::TYPE_SERIALIZED,
			'default' => serialize(array()),
			'verification' => array(
				$this,
				'_verifyExtendedPollOptions'
			),
		);

		return $fields;
	}

	public function addResponses(array $responses)
	{
		if (!empty(ExtendedPoll_Listener::$controller))
		{
			ExtendedPoll_Listener::$controller->actionWithPollDataWriter($this);
		}

		return parent::addResponses($responses);
	}

	protected function _verifyExtendedPollOptions(&$options)
	{
		if (!is_array($options))
		{
			$options = @unserialize($options);
			if (empty($options))
			{
				$options = array();
			}
		}

		if (!empty($options['custom_fields']))
		{
			// use All instead of just Available
			$availableCustomFields = $this->_getPollModel()->ExtendedPoll_getAllCustomFields();
			$customFields = array();

			foreach (array_keys($options['custom_fields']) as $key)
			{
				$customField = $options['custom_fields'][$key];

				if (empty($customField['value']))
				{
					// DO NOT PROCESS THIS CUSTOM FIELD
					unset($options['custom_fields'][$key]);
					continue;
				}

				if (empty($customField['name']) OR !isset($availableCustomFields[$customField['name']]))
				{
					$this->error(new XenForo_Phrase('extendedpoll_not_supported_custom_field'), 'extendedpoll_options');
					return false;
				}

				$customFieldInfo = $availableCustomFields[$customField['name']];

				if ($customFieldInfo['field_type'] == 'single')
				{
					// validates value with single field type
					// probably should use the method in XenForo_Model_UserField instead?
					if (empty($customFieldInfo['fieldChoices'][$customField['value']]))
					{
						$this->error(new XenForo_Phrase('extendedpoll_must_select_valid_value_custom_field_x', array('title' => $customFieldInfo['title'])), 'extendedpoll_options');
						return false;
					}
				}

				$customFields[$customField['name']] = $customField;
			}

			$options['custom_fields'] = array_values($customFields);
		}

		$options = serialize($options);

		return true;
	}

}
