<?php

class ExtendedPoll_Extend_Model_Poll extends XFCP_ExtendedPoll_Extend_Model_Poll
{
	public function preparePoll(array $poll, $canVote, array $viewingUser = null)
	{
		$result = parent::preparePoll($poll, $canVote, $viewingUser);
		$maxAvatarsPerResponse = 5;

		// check for poll permission instead of using $canVote parametters
		// since 0.9.2
		if (!empty($result['canVote']))
		{
			$tmpError = 0;
			$result['canVote'] = $this->canVoteOnPoll($poll, $tmpError, $viewingUser);
		}

		if (!empty($result['responses']))
		{
			$userIds = array();

			// looks for user ids from voters data
			foreach ($result['responses'] as &$response)
			{
				if (!empty($response['voters']))
				{
					$response['votersWithAvatar_userIds'] = array_slice(array_reverse(array_keys($response['voters'])), 0, min(array(
						count($response['voters']),
						$maxAvatarsPerResponse
					)));
					$userIds = array_merge($userIds, $response['votersWithAvatar_userIds']);
				}
			}

			// gets user info from database using found user ids
			if (!empty($userIds) AND !empty($poll['public_votes']))
			{
				//checks if poll is public, since 0.9.2
				$users = $this->getModelFromCache('XenForo_Model_User')->getUsers(array('user_id' => array_unique($userIds)));
			}
			else
			{
				$users = array();
			}

			foreach ($result['responses'] as &$response)
			{
				$response['votersWithAvatar'] = array();
				if (!empty($response['votersWithAvatar_userIds']))
				{
					foreach ($response['votersWithAvatar_userIds'] as $userId)
					{
						if (isset($users[$userId]))
						{
							$response['votersWithAvatar'][$userId] = $users[$userId];
						}
					}
					unset($response['votersWithAvatar_userIds']);
				}
			}
		}

		if (!is_array($result['extendedpoll_options']))
		{
			$result['extendedpoll_options'] = @unserialize($result['extendedpoll_options']);
			if (empty($result['extendedpoll_options']))
			{
				$result['extendedpoll_options'] = array();
			}
		}

		return $result;
	}

	public function voteOnPoll($pollId, $votes, $userId = null, $voteDate = null)
	{
		$result = parent::voteOnPoll($pollId, $votes, $userId, $voteDate);

		if ($result === true)
		{
			// do our job if the vote action is completed
			if (!is_array($votes))
				$votes = array($votes);

			$poll = $this->getPollById($pollId);
			if ($poll['content_type'] == 'thread')
			{
				if ($userId === null)
					$userId = XenForo_Visitor::getUserId();
				$user = $this->getModelFromCache('XenForo_Model_User')->getUserById($userId);
				$responses = $this->preparePollResponsesFromCache($poll['responses']);

				$votedResponses = array();
				foreach ($votes AS $voteResponseId)
				{
					if (isset($responses[$voteResponseId]))
					{
						$votedResponses[$voteResponseId] = $responses[$voteResponseId]['response'];
					}
				}

				$extraData = array(
					'question' => $poll['question'],
					'public_votes' => $poll['public_votes'],
					'votedResponses' => $votedResponses,
				);

				$this->getModelFromCache('XenForo_Model_NewsFeed')->publish($userId, $user['username'], 'extendedpoll_thread', $poll['content_id'], 'extendedpoll_vote', $extraData);
			}
		}

		return $result;
	}

	public function canVoteOnPoll(array $poll, &$errorPhraseKey = '', array $viewingUser = null)
	{
		$result = parent::canVoteOnPoll($poll, $errorPhraseKey, $viewingUser);

		if ($result)
		{
			// only perform our further check if the traditional check return green light
			$this->standardizeViewingUserReference($viewingUser);

			if (!is_array($poll['extendedpoll_options']))
			{
				$poll['extendedpoll_options'] = @unserialize($poll['extendedpoll_options']);
				if (empty($poll['extendedpoll_options']))
					$poll['extendedpoll_options'] = array();
			}

			if (!empty($poll['extendedpoll_options']['custom_fields']))
			{
				// checks custom fields now...
				$hasUserFields = false;
				$userFields = array();
				$availableCustomFields = $this->ExtendedPoll_getAvailableCustomFields();
				$customFields = array();

				foreach ($poll['extendedpoll_options']['custom_fields'] as $customField)
				{
					if (!empty($availableCustomFields[$customField['name']]))
					{
						$customFields[] = $customField;
						if ($availableCustomFields[$customField['name']]['table'] == 'xf_user_field')
							$hasUserFields = true;
					}
				}

				if ($hasUserFields)
				{
					$userFields = $this->getModelFromCache('XenForo_Model_UserField')->getUserFields(array(), array('valueUserId' => $viewingUser['user_id']));
				}

				foreach ($customFields as $customField)
				{
					$customFieldInfo = $availableCustomFields[$customField['name']];

					switch ($customFieldInfo['table'])
					{
						case 'xf_user':
							if (empty($viewingUser[$customField['name']]) OR $viewingUser[$customField['name']] != $customField['value'])
							{
								$errorPhraseKey = $this->ExtendedPoll_getErrorPhrase($customFieldInfo, $customField);
								return false;
							}
							break;
						case 'xf_user_field':
							if (empty($userFields[$customField['name']]) OR empty($userFields[$customField['name']]['field_value']) OR $userFields[$customField['name']]['field_value'] != $customField['value'])
							{
								$errorPhraseKey = $this->ExtendedPoll_getErrorPhrase($customFieldInfo, $customField);
								return false;
							}
							break;
					}
				}
			}
		}

		return $result;
	}

	public function ExtendedPoll_preparePost(array $poll, array &$post)
	{
		if (empty($poll['public_votes']))
		{
			// ignore if poll is not public
			return;
		}

		if (empty($post['user_id']))
		{
			// ignore guests' post
			return;
		}

		$votedResponses = array();

		foreach ($poll['responses'] as $response)
		{
			if (isset($response['voters'][$post['user_id']]))
			{
				// user voted for this response
				$votedResponses[] = $response['response'];
			}
		}

		$post['ExtendedPoll_votedResponses'] = $votedResponses;
	}

	public function ExtendedPoll_getAllCustomFields()
	{
		$customFields = array('gender' => array(
				'name' => 'gender',
				'table' => 'xf_user',
				'title' => new XenForo_Phrase('gender'),
				'field_type' => 'select',
				'fieldChoices' => array(
					'male' => new XenForo_Phrase('male'),
					'female' => new XenForo_Phrase('female'),
				),
			), );

		$userFieldModel = $this->getModelFromCache('XenForo_Model_UserField');
		$userFields = $userFieldModel->getUserFields();
		foreach ($userFields as $userField)
		{
			$userField = $userFieldModel->prepareUserField($userField, true);
			$userField['name'] = $userField['field_id'];
			$userField['table'] = 'xf_user_field';
			$customFields[$userField['name']] = $userField;
		}

		return $customFields;
	}

	public function ExtendedPoll_getAvailableCustomFields()
	{
		static $cached = false;

		if ($cached === false)
		{
			$cached = array();
			$all = $this->ExtendedPoll_getAllCustomFields();
			$option = ExtendedPoll_Option::get('customFields');

			if (!empty($option))
			{
				foreach ($option as $name)
				{
					if (isset($all[$name]))
					{
						$cached[$name] = $all[$name];
					}
				}
			}
		}

		return $cached;
	}

	protected function ExtendedPoll_getErrorPhrase(array $info, $customField)
	{
		if (($info['field_type'] == 'select' OR $info['field_type'] == 'radio') AND !empty($info['fieldChoices']) AND !empty($info['fieldChoices'][$customField['value']]))
		{
			return new XenForo_Phrase('extendedpoll_custom_field_x_must_be_y', array(
				'title' => $info['title'],
				'value' => $info['fieldChoices'][$customField['value']],
			));
		}

		return new XenForo_Phrase('extendedpoll_custom_field_not_matched');
	}

}
